from ..abstractions.Comparable import *


class BinaryTreeNode(Comparable):
    '''
    Basic node of a basic binary tree.
    '''

    def diff(self, other):
        return signum(self.value, other.value)

    def __init__(self, value):
        self.value = value
        self.left = None
        self.right = None
        return

    def compare(self, value, getDirection=False):
        X = self.left if value > self.value else self.right
        return (X, value > self.value) if getDirection else X


class BinaryTree:
    """
    Basic binary tree.
    """

    def __init__(self, nodeType=BinaryTreeNode):
        self.nodeType = nodeType
        self.root: (BinaryTreeNode, None) = None
        self.size = 0
        return

    # -------------------------------------------------------------------------
    # The following functions do not change the data structure.
    def make_node(self, value):
        return self.nodeType(value)

    def __sizeof__(self):
        return self.size

    def __contains__(self, item):
        current = self.root
        while current is not None:
            current: BinaryTreeNode
            if current.value == item:
                return True
            current = current.compare(item)
        return False

    def get(self, item):
        current = self.root
        while current is not None:
            current: BinaryTreeNode
            if current.value == item:
                return current
            current = current.compare(item)
        return False

    # -------------------------------------------------------------------------
    # The following functions DO change the data structure.
    def insert(self, value, log: list = None):
        if self.root is None:
            self.root = self.make_node(value)
            self.size = 1
            return
        current: BinaryTreeNode = self.root
        last: BinaryTreeNode = self.root
        lastJump: bool = False
        while current is not None:
            last = current
            X = last.compare(value, True)
            last = X[0]
            lastJump = X[1]
            if log is not None:
                log.append((last, X[1]))
        new: BinaryTreeNode = self.make_node(value)
        if lastJump:
            last.right = new
        else:
            last.left = new
        return

    def remove(self, value):
        return



def signum(a, b):
    """
    Returns 1 if value of object a is greater than that of object b.
    Returns 0 if objects are equivalent in value.
    Returns -1 if value of object a is lesser than that of object b.
    """
    return int(a > b) - int(a < b)


class Comparable:
    """
    An abstract class that can be inherited to make a class comparable and sortable.
    For proper functionality, function diff must be overridden.
    """

    def diff(self, other):
        """
        Calculates the difference in value between two objects and returns a number.
        If the returned number is
        - positive, the value of object a is greater than that of object b.
        - 0, objects are equivalent in value.
        - negative, value of object a is lesser than that of object b.
        Used in comparison operations.
        Override this function."""
        raise NotImplementedError

    def __eq__(self, other):
        return self.diff(other) == 0

    def __ne__(self, other):
        return self.diff(other) != 0

    def __lt__(self, other):
        return self.diff(other) < 0

    def __le__(self, other):
        return self.diff(other) <= 0

    def __gt__(self, other):
        return self.diff(other) > 0

    def __ge__(self, other):
        return self.diff(other) >= 0


import pathlib
from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent

setup(
    name='potatoutils',
    version='0.0.1',
    packages=find_packages(exclude=['tests*']),
    license='MIT',
    description='',
    long_description=open('README.txt').read(),
    install_requires=[],
    url='',
    author='Dorijan Cirkveni AKA Mouse Potato Does Stuff',
    author_email='mousepotato@tutanota.com'
)
# packagedir={'': 'src'},
# packages=find_packages(exclude=['tests*']),



from .BinaryTree import *


class AVLNode(BinaryTreeNode):
    def __init__(self, value):
        super(AVLNode, self).__init__(value)
        self.depth = 0


class AVLTree(BinaryTree):
    def __init__(self):
        super(AVLTree, self).__init__(AVLNode)
        return

    def insert(self, value, log: list = None):
        super(AVLTree, self).insert(value, log)
        return